# -*- coding: utf-8 -*-
#
##########################################################################
# creole.containers - management of LXC containers
# Copyright © 2012,2013 Pôle de compétences EOLE <eole@ac-dijon.fr>
#
# License CeCILL:
#  * in french: http://www.cecill.info/licences/Licence_CeCILL_V2-fr.html
#  * in english http://www.cecill.info/licences/Licence_CeCILL_V2-en.html
##########################################################################

"""Manage LXC containers

"""

from creole.client import CreoleClient, _CONTAINER_COMPONENTS
from creole.config import VIRTENABLED_LOCKFILE, VIRTDISABLED_LOCKFILE
from creole.error import VirtError
from creole.config import templatedir, VIRTROOT
from creole.template import CreoleTemplateEngine
from pyeole.process import system_code, system_out
from pyeole.diagnose import test_tcp

from distutils.spawn import find_executable
from os.path import isdir, isfile, join
from os import symlink
from os import makedirs

client = CreoleClient()

_NOT_REALLY_LXC_CONTAINERS = ['root', 'all']
"""List of container names that are not to be generated.

"""

_LXC_TEMPLATE = {'config': "lxc.config",
                 'fstab': "lxc.fstab",
                 'rootfs/etc/network/interfaces' : "lxc.interfaces",
                }
"""Creole templates for LXC containers.

"""

def is_lxc_locked():
    """Check if the LXC virtualization is locked.

    The virtualization is locked after first ``instance`` of the
    server to avoid switching between modes.

    :return: ``enable`` if LXC is enabled, ``disable`` if LXC is
             disabled or ``None`` where there is no lockfile.

    """
    if isfile(VIRTENABLED_LOCKFILE) and isfile(VIRTDISABLED_LOCKFILE):
        raise VirtError(u"Invalid LXC lock files state: both are present.")
    elif isfile(VIRTENABLED_LOCKFILE):
        virtlocked = 'enable'
    elif isfile(VIRTDISABLED_LOCKFILE):
        virtlocked = 'disable'
    else:
        virtlocked = None
    return virtlocked

def is_lxc_enabled():
    """Check if LXC controller is enabled

    We do not accept to switch between enabled and disabled LXC, after
    first ``instance``, a lock file is set to check at each
    ``reconfigure``.

    :return: If the LXC container mode is enabled.
    :rtype: `bool`
    :raise VirtError: if state in inconsistent between configuration
                      and lock files.

    """
    containers_enabled = client.get_creole('mode_conteneur_actif', 'non') == 'oui'
    if containers_enabled and not find_executable('lxc-info'):
        raise VirtError(u'LXC is enabled but LXC commands not found in PATH.')

    if containers_enabled and is_lxc_locked() == 'disable':
        raise VirtError(u"LXC is enabled but disabled lock is present.")
    elif not containers_enabled and is_lxc_locked() == 'enable':
        raise VirtError(u"LXC is disabled but enabled lock is present.")

    return containers_enabled

def generate_lxc_container(name):
    """Run creation of a container.

    Check if LXC is enabled and take care of ``root`` and ``all``
    containers.

    :param name: name of the LXC container
    :type name: `str`

    """
    if is_lxc_enabled() and name not in _NOT_REALLY_LXC_CONTAINERS:
        system_code(['lxc-create', '-n', name, '-t', 'eole', '--',
                     '--release', 'precise'])
        auto_file = '/etc/lxc/auto/{0}.conf'.format(name)
        if not isfile(auto_file):
            symlink('/var/lib/lxc/{0}/config'.format(name), auto_file)

def generate_lxc_containers():
    """Run creation of all containers.

    Only the container groups are generated.

    """
    groups = client.get_groups()

    for name in groups:
        generate_lxc_container(name)

def is_lxc_running(container):
    """Check if an LXC container is running.

    This check at LXC level and check TCP on port SSH.

    :param container: the container informations
    :type container: `dict`
    :return: if the container is running and reachable
    :rtype: `bool`

    """

    if container.get(u'name', None) is None:
        raise ValueError(u"Container has no name")

    if container.get(u'ip', None) is None:
        raise ValueError(u"Container {0} has no IP".format(container[u'name']))

    cmd = ['lxc-info', '--state', '--name', container[u'name']]
    code, stdout, stderr = system_out(cmd)

    return stdout.strip().endswith('RUNNING') and test_tcp(container[u'ip'], 22)


class ContainerTemplateEngine(CreoleTemplateEngine):
    """Engine to process LXC configuration files.

    """

    def instance_lxc_file(self, filename, group, minimal=False):
        """Run templatisation of one container configuration

        :param filename: name of the configuration file as defined in
                         keys of :data:`_LXC_TEMPLATE`
        :type filename: `str`
        :param group: name of the group of containers
        :type group: `str`
        :param minimal: strip extra informations like interfaces for a
                        minimal setup
        :type minimal: `bool`

        """
        if group in _NOT_REALLY_LXC_CONTAINERS:
            raise VirtError(u"Can not generate configuration for non"
                            u" LXC containers '{0}'".format(group))

        lxc_conf = { 'full_name': join(VIRTROOT, group, filename),
                     'name': join(VIRTROOT, group, filename),
                     'source': join(templatedir,
                                     _LXC_TEMPLATE[filename].lstrip('/')),
                     'activate': True,
                     'del_comment': u'',
                     'mkdir' : False,
                     'rm' : False,
        }

        # container context for CheetahTemplate
        group = client.get_group_infos(group)
        # Force chrooting on the root
        group['chroot'] = '/'

        if minimal:
            # Keep only interface 'containers'
            minimal_interfaces = []
            for interface in group['interfaces']:
                if interface['name'] != 'containers':
                    continue
                group['interfaces'] = [interface]
                group['gateway'] = {u'interface': interface['name'],
                                    u'ip':interface['gateway']}
                break

        self._instance_file(lxc_conf, group)
        self._create_mount_points(group)

    def instance_lxc_files(self, group=None, minimal=False):
        """Run templatisation of all containers configurations

        :param group: name of the group of containers
        :type group: `str`
        :param minimal: strip extra informations like interfaces for a
                        minimal setup
        :type minimal: `bool`

        """
        if group is None:
            groups = client.get_groups()
        else:
            groups = [client.get_container(group)['name']]

        for group in groups:
            if group in _NOT_REALLY_LXC_CONTAINERS:
                continue
            for template in _LXC_TEMPLATE.keys():
                self.instance_lxc_file(template, group, minimal=minimal)

    def _create_mount_points(self, group):
        """Create mount points in LXC.

        This is required for LXC to start.

        :param group: informations of the containers group
        :type group: `dict`

        """
        if 'fstabs' not in group:
            return True
        for fstab in group['fstabs']:
            full_path = join(group['path'], fstab['mount_point'].lstrip('/'))
            if not isdir(full_path):
                makedirs(full_path)

    def instance_file(self, *args, **kwargs):
        """Disabled method.

        """
        raise NotImplementedError(u"This method is valid only in CreoleTemplateEngine")

    def instance_files(self, *args, **kwargs):
        """Disabled method.

        """
        raise NotImplementedError(u"This method is valid only in CreoleTemplateEngine")
