# -*- coding: utf-8 -*-

"""Apply configuration of EOLE servers.

"""

import os
import argparse
import time
import shutil

from glob import glob
from shutil import move

import spwd
import getpass
from passlib.context import CryptContext
from itertools import count

from pyeole.log import getLogger
from pyeole.log import init_logging
from pyeole.log import set_formatter
from pyeole.log import set_filters

from pyeole import scriptargs
from pyeole.lock import acquire, release
from pyeole import process
from pyeole.schedule import display_schedules, apply_schedules
from pyeole import ihm
from pyeole.pkg import EolePkg, PackageNotFoundError, \
        _configure_sources_eole, _configure_sources_ubuntu
from pyeole.service import manage_service
from pyeole.service import unmanaged_service
from pyeole.service import manage_services

from creole.error import FileNotFound, LockError, UnlockError
from creole.error import UserExit, UserExitError
from creole.error import VirtError
from creole.client import CreoleClient, CreoleClientError
from creole import fonctionseole, template, cert
from creole.eosfunc import is_instanciate
from creole.config import configeol, INSTANCE_LOCKFILE
from creole.containers import is_lxc_enabled
from creole.containers import generate_lxc_containers
from creole.containers import is_lxc_running
from creole.containers import ContainerTemplateEngine
from creole.error import NetworkConfigError

try:
    from zephir.lib_zephir import lock, unlock
    zephir_libs = True
except Exception:
    zephir_libs = False

client = CreoleClient()

global PKGMGR
PKGMGR = None

error_msg_documentation = """Pour plus d'informations consulter : la documentation du module ou
la documentation commune 'Mise en oeuvre des modules EOLE'."""
def load_pkgmgr():
    global PKGMGR
    if PKGMGR is None:
        PKGMGR = EolePkg('apt')

_LOGFILENAME = '/var/log/reconfigure.log'

# Command line options
class Option:
    """Manage commande line options with defaults

    """
    def __init__(self):
        self.parser = argparse.ArgumentParser(
                description=u"Application de la configuration EOLE",
                parents=[scriptargs.container(),
                scriptargs.logging(level='info')])
        self.parser.add_argument('-i', '--interactive', action='store_true',
                help=u"lancer le processus en mode interactif")
        self.parser.add_argument('-f', '--force', action='store_true',
                help=u"outrepasser le blocage par Zéphir")
        self.parser.add_argument('-a', '--auto', action='store_true',
                help=u"redémarrage automatique si nécessaire")
        self.__opts = self.parser.parse_args([])

    def update_from_cmdline(self, force_args=None, force_options=None):
        """Parse command line
        """
        self.__opts = self.parser.parse_args(force_args)
        if self.__opts.verbose:
            self.__opts.log_level = 'info'
        if self.__opts.debug:
            self.__opts.log_level = 'debug'
        if force_options is not None:
            for key, value in force_options.items():
                setattr(self.__opts, key, value)
        self.__dict__.update(self.__opts.__dict__)

    def __getattr__(self, name):
        if name in ['__opts', 'update_from_cmdline']:
            return self.__dict__[name]
        else:
            return getattr(self.__opts, name)

options = Option()

# To use log from every functions
log = getLogger(__name__)

# Same name for instance and reconfigure
LOCK_NAME = u'reconfigure'

# Run scripts in directories
RUNPARTS_PATH = u'/usr/share/eole'
RUNPARTS_CMD = u'/bin/run-parts --exit-on-error -v {directory} --arg {compat}'

# Compatibility
COMPAT_NAME = u'reconfigure'

#def parse_cmdline():
#    """Parse command line
#    """
#    descr = u"Application de la configuration EOLE"
#    parser = argparse.ArgumentParser(description=descr,
#                                     parents=[scriptargs.container(),
#                                              scriptargs.logging(level='info')])
#    parser.add_argument('-i', '--interactive', action='store_true',
#                        help=u"lancer le processus en mode interactif")
#    parser.add_argument('-f', '--force', action='store_true',
#                        help=u"force l'action même s'il existe des verrous")
#    parser.add_argument('-a', '--auto', action='store_true',
#                        help=u"redémarrage automatique si nécessaire")
#
#    opts = parser.parse_args()
#    if opts.verbose:
#        opts.log_level = 'info'
#    if opts.debug:
#        opts.log_level = 'debug'
#    return opts

def copyDirectoryContent(src, dst):
    for fic in os.listdir(src):
        try:
            shutil.copy2(os.path.join(src, fic), dst)
        except shutil.Error, err:
            # ignore if files already exists
            pass

def user_exit(*args, **kwargs):
    """
    sortie utilisateur "propre"
    """
    log.warn(u'! Abandon de la configuration !')
    log.warn(u'Le système peut être dans un état incohérent\n\n')
    raise UserExitError()

def unlock_actions():
    if zephir_libs:
        #FIXME: lock de Zephir !
        unlock('actions')
    try:
        release(LOCK_NAME, level='system')
    except Exception, err:
        # FIXME: move lock exception to pyeole.lock #7400
        raise UnlockError(err.message)

def lock_actions():
    try:
        acquire(LOCK_NAME, level="system")
    except Exception, err:
        # FIXME: move lock exception to pyeole.lock #7400
        raise LockError(err.message)
    if zephir_libs:
        #FIXME: lock de Zephir !
        lock('actions')

def reset_compat_name():
    """
    Réinitialise le nom de la procédure en cours
    en fonction de l'environnement
    """
    global COMPAT_NAME
    if options.interactive:
        COMPAT_NAME = u'instance'
    else:
        COMPAT_NAME = u'reconfigure'

def run_parts(directory):
    """Run scripts in a directory

    @param directory: name of a directory
    @type directory: C{str}
    """
    dirpath = os.path.join(RUNPARTS_PATH, directory)
    if os.path.isdir(dirpath):
        ihm.print_title(u'Exécution des scripts {0}'.format(directory))
        code = os.system(RUNPARTS_CMD.format(directory=dirpath, compat=COMPAT_NAME))
        if code != 0:
            raise Exception(u'Erreur {0}'.format(directory))

def prepare(need_lock=True):
    """Sanity checks.

    """
    global RUNPARTS_CMD
    # Clean exit
    if need_lock:
        fonctionseole.catch_signal(user_exit)
        lock_actions()

    if options.container != None:
        RUNPARTS_CMD += u" --regex '^[09][09]-{0}$'".format(options.container)

    ihm.print_title(u"Préparation pour {0}".format(COMPAT_NAME))

    if not os.path.isfile(configeol):
        print "Le serveur n'est pas configuré."
        print
        print error_msg_documentation
        print
        raise FileNotFound(u'Fichier {0} manquant.'.format(configeol))

    display_info = False

    if not options.interactive and is_instanciate() == 'non':
        ihm.print_red("Le serveur n'a jamais été instancié alors que "
                         "c'est une étape obligatoire avant la reconfiguration.")
        display_info = True

    if options.interactive and is_instanciate() == 'oui':
        ihm.print_red("Le serveur est déjà instancié")
        print
        print "Pour modifier un paramètre de configuration (ex.: adresse IP) utiliser :"
        print "'gen_config'"
        print "et 'reconfigure' pour appliquer les modifications."
        display_info = True

    if display_info:
        print
        print error_msg_documentation
        print
        if not options.interactive:
            raise Exception("Instancier d'abord le serveur.")
        else:
            if ihm.question_ouinon(u"Continuer instanciation quand même ?",
                                   interactive=options.interactive,
                                   default=u'non') == u'non':
                raise UserExit()
            else:
                fonctionseole.zephir("MSG", "Instance forcée par l'utilisateur",
                                     COMPAT_NAME.upper())

    unmanaged_service(u'stop', u'creoled', u'service', display='console')
    unmanaged_service(u'start', u'creoled', u'service', display='console')

    try:
        client.get_creole(u'eole_version')
    except CreoleClientError:
        msg = u"Impossible d’accéder aux variables Creole: voir /var/log/creoled.log"
        raise CreoleClientError(msg)

    if fonctionseole.init_proc(COMPAT_NAME.upper()) == False and not options.force:
        log.warn(u"Cette procédure est bloquée, Contacter l'administrateur Zephir")
        if ihm.question_ouinon(u"Forcer l'exécution ?",
                               interactive=options.interactive,
                               default=u'non') == u'non':
            if not options.interactive:
                log.warn(u"Utilisez l'option -f si vous désirez forcer l'exécution")
            raise UserExitError()
        else:
            fonctionseole.zephir("MSG",
                                 "Instance forcée par l'utilisateur",
                                 COMPAT_NAME.upper())


def valid_mandatory():
    client.valid_mandatory()

def containers(minimal=True):
    """Generate containers
    """
    VAR_LXC='/var/lib/lxc'
    OPT_LXC='/opt/lxc'

    if not is_lxc_enabled():
        log.debug(u'Le mode conteneur est désactivé.')
        return True

    # make /var/lib/lxc -> /opt/lxc
    if os.path.isdir(VAR_LXC) and not os.path.exists(OPT_LXC):
        ihm.print_title(u"Mise en place de {0}".format(OPT_LXC))
        manage_service(u'stop', u'lxc', display='console')
        move(VAR_LXC, OPT_LXC)
        os.symlink(OPT_LXC, VAR_LXC)

    ihm.print_title(u'Génération des conteneurs')
    generate_lxc_containers()

    ihm.print_title(u'Génération des configurations minimales des conteneurs.')
    lxc_engine = ContainerTemplateEngine()
    lxc_engine.instance_lxc_files(minimal=minimal)

    ihm.print_title(u'Ajout des conteneurs à la configuration SSH.')
    engine = template.CreoleTemplateEngine()
    engine.instance_file(u'/etc/ssh/ssh_config')

    if minimal:
        # Stop all containers before remove/install of package
        # to be sure the next start will start them all
        manage_service(u'stop', u'lxc', display='console')
    else:
        # starting LXC with their interfaces requires a good network
        manage_service(u'restart', u'networking', display='console')

    # Make sure br0 is UP
    unmanaged_service(u'stop', u'lxc-net', u'upstart', display='console')
    unmanaged_service(u'start', u'lxc-net', u'upstart', display='console')

    manage_service(u'start', u'lxc', display='console')

    # Wait for running LXC
    lxc_names = client.get_groups()
    lxc_names.remove(u'root')
    groups = [client.get_container(group) for group in lxc_names]

    max_try=10
    for count in range(max_try):
        running = 0
        not_running = []

        for group in groups:
            if is_lxc_running(group):
                running += 1
            else:
                not_running.append(group['name'])

        waiting_for = ', '.join(not_running)
        if minimal and count == max_try - 1 and running != len(lxc_names):
            msg = u"Unable to start LXC container{0}: {1}"
            raise VirtError(msg.format(u's' * (len(not_running) > 1),
                                       waiting_for))
        elif running != len(lxc_names):
            msg = u"Waiting 1 second for non running LXC: {0}".format(waiting_for)
            log.warning(msg)
            time.sleep(1)
        else:
            break


def remove_packages():
    """ Remove packages listed in /usr/share/eole/remove.d/ files
        param: repo: EoleApt Object
    """
    torm_conf = glob(u'/usr/share/eole/remove.d/*.conf')
    pkg_list = []
    for config in torm_conf:
        try:
            f_h = open(config, 'r')
            for line in f_h.readlines():
                pkg_list.append(line.strip('\n'))
            f_h.close()
        except IOError, err:
            log.error(u'Can not read file {0}: {1}'.format(config,
                                                           err.message))
    kernels = fonctionseole.get_kernel_to_remove()

    if kernels:
        ihm.print_line("Suppression des anciens noyaux")
        pkg_list.extend(kernels)

    if pkg_list != []:
        try:
            load_pkgmgr()
            PKGMGR.remove(packages=pkg_list)
        except (PackageNotFoundError, SystemError), err:
            msg = u'Impossible de supprimer certains paquets: {0}'
            log.warn(msg.format(err.message))
            log.warn(u'Ces paquets seront supprimés au prochain reconfigure')


def install_packages():
    """Install package for each container group
    """
    load_pkgmgr()
    for group_name in client.get_groups():
        group_infos = client.get_group_infos(group_name)
        package_names = [pkg[u'name'] for pkg in group_infos[u'packages']]
        if package_names != []:
            PKGMGR.install(packages=package_names,
                           container=group_infos[u'name'])


def packages():
    """Manage packages
    """
    ihm.print_title(u'Gestion des paquets')
    log.info(u'   Suppression des paquets')
    ihm.print_line(u'Suppression des paquets')
    remove_packages()
    log.info(u'   Installation des paquets')
    ihm.print_line(u'Installation des paquets')
    install_packages()


def templates():
    """Run pretemplate scripts and manage templates
    """
    run_parts(u'pretemplate')
    ihm.print_title(u'Génération des fichiers de configuration')
    log.info(u'Génération des fichiers de configuration')
    try:
        tmpl = template.CreoleTemplateEngine()
        tmpl.instance_files(container=options.container)
    except Exception, err:
        # FIXME: do we really need to rewrite the message ?
        #msg = u"Erreur lors de l'instanciation des templates : {0}"
        #log.debug(msg.format(err.message), exc_info=True)
        # hack the original message
        #err.message = msg.format(err.message)
        raise err


def services(action):
    """Manage services
    """
    exclude = None
    if action == u'stop':
        ihm.print_title(u"Arrêt des services")
        exclude = (('root', 'networking'),)
    elif action == u'start':
        ihm.print_title(u"Démarrage des services")
    elif action == u'configure':
        ihm.print_title(u"Configuration des services")
    elif action == u'start':
        ihm.print_title(u"Démarrage/Arrêt des services")
    else:
        raise ValueError(u"Action de service inconnue: {0}".format(action))

    if not is_lxc_enabled():
        manage_services(action, container=options.container, display='console', exclude=exclude)
    else:
        if action != u'configure' and options.container is None:
            # Limit to root services as we stop/start LXC themself
            container='root'
        else:
            container=options.container
        manage_services(action, container=container, display='console', exclude=exclude)


def _gen_user_list():
    """Generate list of users for password modification

    Start with basic one and ask for supplementary users.
    """
    yield 'root'

    for number in count(1):
        if number == 1:
            yield 'eole'
        else:
            yield 'eole{0}'.format(number)

def users():
    """Manage users
    """
    ihm.print_title(u'Gestion des utilisateurs systèmes')
    schemes = [u'sha512_crypt', u'sha256_crypt', u'sha1_crypt', u'md5_crypt']
    cryptctx = CryptContext(schemes=schemes)
    default_pass = {u'root': u'$eole&123456$',
                    u'eole': u'$fpmf&123456$'}

    if not options.interactive:
        log.debug(u'Aucune gestion utilisateur en mode non interactif')
        return

    for user in _gen_user_list():
        try:
            user_infos = spwd.getspnam(user)
        except KeyError:
            if user == u'root':
                msg = u"Utilisateur « root » inconnu, ce n’est pas normal"
                raise Exception(msg)

            prompt = 'Créer un nouvel administrateur %s ?'
            if user != 'eole' and ihm.question_ouinon(prompt % user) == 'non':
                break

            msg = u"Création de l’utilisateur inexistant {0}"
            log.info(msg.format(user))

            cmd = ['adduser', '--quiet', '--shell', '/usr/bin/manage-eole',
                   '--gecos', '{0} user'.format(user.upper()),
                   '--disabled-password', user]
            code = process.system_code(cmd)
            if code != 0:
                msg = u"Impossible de créer l’utilisateur {0}"
                raise Exception(msg.format(user))

            cmd = ['adduser', '--quiet', user, 'adm']
            code, out, err = process.system_out(cmd)
            if code != 0:
                msg = u"Impossible d’ajouter {0} au groupe adm"
                raise Exception(msg.format(user))

            # Update informations
            user_infos = spwd.getspnam(user)

        if user not in default_pass and user_infos.sp_pwd not in ['!', '*']:
            msg = u"Pas de changement de mot de passe de l’administrateur supplémentaire {0}"
            log.warn(msg.format(user))
            continue

        if (user not in default_pass or user_infos.sp_pwd in ['!', '*']
            or cryptctx.verify(default_pass[user], user_infos.sp_pwd)):

            msg = u"# Changement du mot de passe pour l’utilisateur {0} #"
            msg = msg.format(user)
            log.warn(u'#' * len(msg))
            log.warn(msg)
            log.warn(u'#' * len(msg))
            max_try = 5
            prompt = u'{0}{1}: '
            first_prompt = u"Nouveau mot de passe"
            second_prompt = u"Confirmation du mot de passe"
            loop_counter = u''
            for attempt in range(1, max_try+2):
                if attempt == max_try+1:
                    msg = u"Erreurs de saisie de mot de passe de {0}, abandon."
                    raise Exception(msg.format(user))

                loop_counter = loop_counter.format(attempt, max_try)
                passwd = getpass.getpass(prompt.format(first_prompt,
                                                       loop_counter))
                confirm_pass = getpass.getpass(prompt.format(second_prompt,
                                                             loop_counter))
                if passwd == confirm_pass:
                    if user in default_pass and default_pass[user] == passwd:
                        log.error(u"Impossible d'utiliser le mot de passe par défaut.")
                    else:
                        # Now we have the password
                        stdin = '{0}:{1}'.format(user, passwd)
                        code, stdout, stderr = process.system_out(['chpasswd'],
                                                                  stdin=stdin)
                        if code == 0:
                            msg = u'Mot de passe de l’utilisateur {0} mis à jour.'
                            log.info(msg.format(user))
                            # Success
                            break
                        msg = u"Erreur de changement de mot de passe de {0}."
                        try_again_pos = stdout.find('Try again.')
                        chpassmsg = stdout[0:try_again_pos]
                        log.error(msg.format(user))
                        print chpassmsg
                else:
                    log.error(u"Les mots de passe ne correspondent pas.")

                # Display counter
                loop_counter = u' ({0}/{1})'


def certificates():
    """Manage certificates

    """
    ihm.print_title(u'Gestion des certificats')
    try:
        cert.gen_certs()
        # regénération des hashes des certificats SSL après avec créé les nouveaux certificats
        # porté de 2.3 #8488
        process.system_out(['/usr/bin/c_rehash'])
    except Exception, err:
        raise "Erreur lors de la génération des certificats : {0}".format(err)

    if is_lxc_enabled():
        lxc_names = client.get_groups()
        lxc_names.remove(u'root')
        rootPath = client.get_container(u'root')["path"]
        src = os.path.join('/', rootPath.lstrip('/'), cert.ssl_dir.lstrip('/'), "certs")
        groups = [client.get_container(group) for group in lxc_names]
        for group in groups:
            ihm.print_line("Copie des certificats dans : {0}".format(group['name']))
            dst = os.path.join('/', group['path'].lstrip('/'), src.lstrip('/'))
            copyDirectoryContent(src, dst)


def param_kernel():
    """Manage kernel parameters
    """
    ihm.print_title(u'Application des paramètres Noyau')
    os.system('/sbin/sysctl -p >/dev/null')

def kill_dhclient():
    """Kill dhclient for static IP configuration.

    """
    if client.get_creole(u'eth0_method') == u'statique':
        os.system('pkill dhclient3 2>/dev/null')

def finalize(need_lock=True):
    """Clean up
    """
    ihm.print_title(u'Finalisation de la configuration')
    # enregistrement
    try:
        fonctionseole.register_module(ask=options.interactive)
    except Exception:
        pass
    fonctionseole.zephir("FIN", "Configuration terminée", COMPAT_NAME.upper())
    if not os.path.isfile(INSTANCE_LOCKFILE):
        # l'instance est allée à son terme (#7051)
        file(INSTANCE_LOCKFILE, 'w').close()
    if need_lock:
        unlock_actions()

def update_server():
    """Manage server update
    """
    if options.interactive:
        log.info(u'Gestion de la mise à jour')

        ihm.print_title(u'Mise à jour du serveur')
        if ihm.question_ouinon(u"""Une mise à jour est recommandée
Voulez-vous effectuer une mise à jour via le réseau maintenant ?""",
                               default=u'oui', level='warn',
                               default_uninteractive=u'non') == u'oui':
            load_pkgmgr()
            _configure_sources_ubuntu(PKGMGR)
            _configure_sources_eole(PKGMGR)
            PKGMGR.update(silent=True)
            upgrades = PKGMGR.get_upgradable_list(silent=True)
            require_dist_upgrade = False
            for container, upgrades in upgrades.items():
                if upgrades:
                    require_dist_upgrade = True
                    break
            if require_dist_upgrade:
                # At least one container require upgrade
                PKGMGR.dist_upgrade()
                # recall reconfigure
                main(force_options={'interactive': False})
                # back to instance
                options.interactive = True
                reset_compat_name()
            else:
                log.warn(u"Aucune mise à jour à effectuer.")


def schedule():
    """Manage task scheduling
    """
    ihm.print_title(u'Planificateur de tâche')
    apply_schedules()
    display_schedules()
    # 1er lancement de instance
    #if not os.path.isfile(schedule.SCHEDULE_FILE):
    #    schedule.add_post_schedule('majauto', 'weekly')
    #schedule.prog_schedule()

def is_valid_ip_eth0():
    """Check if adresse_ip_eth0 is 169.254.0.1
    """
    ip_eth0 = client.get_creole(u'adresse_ip_eth0')
    if ip_eth0 == "169.254.0.1":
        return False
    else:
        return True


def reboot_server():
    """Reboot the server if required
    """
    if fonctionseole.controle_kernel():
        if options.interactive:
            print
            if ihm.question_ouinon(u"""Un redémarrage est nécessaire
Faut-il l'effectuer maintenant ?""",
                                   default='oui', level='warn') == 'oui':
                fonctionseole.zephir("MSG",
                                     "Demande de redémarrage acceptée par l'utilisateur",
                                     COMPAT_NAME.upper())
                process.system_code(['reboot'])
            else:
                fonctionseole.zephir("MSG",
                                     "Demande de redémarrage refusée par l'utilisateur",
                                     COMPAT_NAME.upper())
        else:
            print
            ihm.print_orange(u'Reboot nécessaire')
            time.sleep(1)
            print
            if options.auto:
                fonctionseole.zephir("MSG", "Redémarrage automatique",
                                     COMPAT_NAME.upper())
                process.system_code(['reboot'])
            else:
                fonctionseole.zephir("MSG", "Redémarrage du serveur à planifier",
                                     COMPAT_NAME.upper())


def main(force_options=None, force_args=None, need_lock=True):
    """Entry point
    """
    global log
    options.update_from_cmdline(force_args=force_args,
                                force_options=force_options)

    try:
        # module level logger
        log = init_logging(name=u'reconfigure', level=options.log_level,
                           console=['stderr', 'stddebug'],
                           filename=_LOGFILENAME)

        # Remove module name prefix from Warn/error messages emitted
        # from here
        set_formatter(log, 'stderr', 'brief')

        # Define handlers for additional loggers
        # Thoses logger are not for use
        # Log pyeole.service
        pyeole_service_log = init_logging(name=u'pyeole.service',
                                          level=options.log_level,
                                          filename=_LOGFILENAME,
                                          console=['stderr'])
        # Disable warnings from pyeole.service
        set_filters(pyeole_service_log, 'stderr',
                    ['error', 'critical'])

        if options.verbose or options.debug:
            # Enable creole logs
            creole_log = init_logging(name=u'creole', level=options.log_level,
                                      filename=_LOGFILENAME)
            # Define a root logger when verbose or debug is activated
            root_log = init_logging(level=options.log_level)
        else:
            # Enable creole logs
            creole_log = init_logging(name=u'creole', level=options.log_level,
                                      filename=_LOGFILENAME,
                                      console=['stderr'])

        creolemajauto_log = init_logging(name=u'creole.majauto', level=options.log_level,
                                  filename=_LOGFILENAME, console=['stderr', 'stdout'])

        ihm.print_title(u'Début de configuration')
        ihm.clear()
        # instance or reconfigure ?
        reset_compat_name()
        fonctionseole.zephir("INIT", "Début de configuration",
                             COMPAT_NAME.upper())
        prepare(need_lock)
        valid_mandatory()
        containers(minimal=True)
        packages()
        run_parts(u'preservice')
        services(action=u'stop')
        templates()
        if not is_valid_ip_eth0():
            log.info(u"La carte eth0 ne dispose pas d'une adresse IP valide")
            log.info(u"Relance du service réseau")
            manage_service(u'restart', u'networking', display='console')
            templates()
            if not is_valid_ip_eth0():
                log.info(u"La carte eth0 ne dispose pas d'une adresse IP valide")
                msg = u"Impossible d'obtenir une adresse IP"
                raise NetworkConfigError(msg)

        services(action=u'configure')
        # posttemplate/00-annuaire needs the certificates
        certificates()
        # Generate containers with all interfaces
        containers(minimal=False)
        run_parts(u'posttemplate')
        param_kernel()
        kill_dhclient()
        services(action=u'start')
        users()
        run_parts(u'postservice')
        schedule()
        finalize(need_lock)
        ihm.print_title(u'Reconfiguration OK')
        update_server()
        # IMPORTANT : Ne rien faire après ces lignes
        # car le serveur est susceptible d'être redémarré
        reboot_server()

    except (UserExit, UserExitError), err:
        unlock_actions()
        fonctionseole.zephir("FIN", "Abandon par l'utilisateur",
                             COMPAT_NAME.upper())
        raise err

    except Exception, err:
        if options.debug:
            log.debug(err.message, exc_info=True)
        else:
            log.error(err.message)
        fonctionseole.zephir('ERR', err.message,
                             COMPAT_NAME.upper(),
                             console=False)
        if need_lock:
            release(LOCK_NAME, valid=False, level='system')
        raise err

if __name__ == '__main__':
    main()
